package TJournal::GUI;

# GUI.pm -- author: Oliver Bossert
# Copyright (C) 2004 Oliver Bossert
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;

use TJournal::Conf;
use TJournal::DB;
use TJournal::Translate;

use Tk;
use Tk::ToolBar;
use Tk::NoteBook;
use Tk::TableMatrix;
use Tk::Adjuster;
use Tk::ROText;
use Tk::Pane;
use Tk::BrowseListbox;

BEGIN {
  unless (eval "use Tk::PNG") {
    warn "Could not load PNG-module...";
  }
}

#use GT::Conf;
#use GT::Eval;

TJournal::Conf::default("Data::FilterFile", "./filters");

TJournal::Conf::default("PFAnalysis::Color::good", "#dcffc8" );
TJournal::Conf::default("PFAnalysis::Color::verygood", "#6dff1e" );
TJournal::Conf::default("PFAnalysis::Color::bad", "#ffdcc8" );
TJournal::Conf::default("PFAnalysis::Color::verybad", "#ff6d1e" );
TJournal::Conf::default("PFAnalysis::Display", "ProfitFactor|PRR|WinRatio|NbGains|NbLosses|AvgGain|AvgLoss|".
			"SumGain|SumLoss|CumGain|CumLoss");

TJournal::Conf::set("PFAnalysis::DisplayAll", "ProfitFactor|PRR|WinRatio|NbGains|NbLosses|AvgGain|AvgLoss|".
		    "SumGain|SumLoss|CumGain|CumLoss");

############################################################
sub new { # Generate a new Object
############################################################
  my $type = shift;
  my $class = ref($type) || $type;
  my $self = {};
  bless $self, $class;

  $self->{'data'} = shift;

# Enable this for the use with GT:
#  GT::Conf::load();
#  $self->{'db'} = create_standard_object("DB::" . GT::Conf::get("DB::module"));

  $self->{'db'} = TJournal::DB->new();

  return $self;
}

############################################################
sub display { # Do all the GUI-stuff
############################################################
  my $self = shift;
  $self->{'mw'} = MainWindow->new(-title=>"Olf's Trading Journal");

  # Create the Menue
  $self->create_menue();

  # Create the Toolbar
  $self->create_toolbar();

  # Create the Filter Menue
  $self->create_filter();

  # Create NoteBook
  $self->create_notebook();

  # Create Overview-Table
  $self->create_table_overview();

  # Create Details-Page
  $self->create_detail_page();

  # Create Analysis-Page
  $self->create_analysis_trade_page();

  # Create Analysis-Page
  $self->create_analysis_portfolio_page();

  MainLoop;
}

############################################################
sub create_menue {
############################################################
  my $self = shift;
  $self->{'mw'}->configure(-menu => $self->{menubar} = $self->{'mw'}->Menu);
  $self->{menubar}->cascade(
			    -label => _t('~Portfolio'),
			    -menuitems => $self->add_menuitems_portfolio()
			   );
  $self->{menubar}->cascade(
			    -label => _t('~View'),
			    -menuitems => $self->add_menuitems_view()
			   );
  $self->{menubar}->cascade(
			    -label => _t('~Configure'),
			    -menuitems => $self->add_menuitems_config()
			   );
}

################################################################################
sub add_menuitems_portfolio {
################################################################################
  my $self = shift;
  $self->{mw}->bind($self->{mw},
		    "<Control-n>" => 
		    sub { $self->portfolio_new(); } );
  $self->{mw}->bind($self->{mw},
		    "<Control-o>" => 
		    sub { $self->portfolio_open(); } );
  $self->{mw}->bind($self->{mw},
		    "<Control-u>" => 
		    sub { $self->portfolio_save_as(); } );
  $self->{mw}->bind($self->{mw},
		    "<Control-q>" => 
		    sub { exit; } );

  [
   [ 'command', _t("~New"), qw/-accelerator Ctrl-n -command/ => sub { $self->portfolio_new(); } ],
   [ 'command', _t("~Open"), qw/-accelerator Ctrl-o -command/ => sub { $self->portfolio_open(); } ],
   [ 'command', _t('S~ave As'), qw/-accelerator Ctrl-u -command/ => sub { $self->portfolio_save_as(); } ],
   '',
   [ 'cascade', _t("E~xport"), qw/-tearoff 0 -menuitems/ =>
     [
      [ 'command', _t('CSV Data'), -command => sub { $self->portfolio_export_csv(); } ],
     ],
   ],
   [ 'cascade', _t("S~ynchronize"), qw/-tearoff 0 -menuitems/ =>
     [
      [ 'command', _t('CSV Data'), -command => sub { $self->portfolio_sync_csv(); } ],
      [ 'command', _t('Depot XML'), -command => sub { $self->portfolio_sync_depot(); } ],
     ],
   ],
   '',
   [ 'command', _t('~Quit'), qw/-accelerator Ctrl-q -command/ => sub { exit } ],
  ];
}

################################################################################
sub add_menuitems_view {
################################################################################
  my $self = shift;
  [
   [ 'command', _t('~Overview'), -command => sub { $self->{'notebook'}->raise("overview"); } ],
   [ 'command', _t('~Details'), -command => sub { $self->{'notebook'}->raise("details"); } ],
   [ 'command', _t('Analysis ~Trade'), -command => sub { $self->{'notebook'}->raise("analysis_trade"); } ],
   [ 'command', _t('Analysis ~Portfolio'), -command => sub { $self->{'notebook'}->raise("analysis_portfolio"); } ],
  ];
}

################################################################################
sub add_menuitems_config {
################################################################################
  my $self = shift;
  [
   [ 'command', _t('~General Configuration'), -command => sub { $self->configure_main(); } ],
   [ 'command', _t('~Filter Configuration'), -command => sub { $self->configure_filter(); } ],
   [ 'command', _t('~Portfolio Analysis Configuration'), -command => sub { $self->configure_portfolio(); } ],
  ];
}

############################################################
sub create_notebook {
############################################################
  my $self = shift;

  $self->{'notebook'} = $self->{'mw'}->NoteBook()->pack( -expand => 1,
							 -fill => 'both' );

  $self->{'nb_overview'} = $self->{'notebook'}->add("overview", 
						    -label => _t("Overview") );

  $self->{'nb_details'} = $self->{'notebook'}->add("details", 
						    -label => _t("Details"),
						   -raisecmd => sub { $self->update_detail_page } );

  $self->{'nb_analysis_trade'} = $self->{'notebook'}->add("analysis_trade", 
							  -label => _t("Analysis Trade"),
							  -raisecmd => sub { $self->update_analysis_trade_page() }  );

  $self->{'nb_analysis_portfolio'} = $self->{'notebook'}->add("analysis_portfolio", 
							      -label => _t("Analysis Portfolio"),
							      -raisecmd => sub { $self->update_portfolio_trade_page() }  );

}


############################################################
sub create_toolbar {
############################################################
  my $self = shift;
  $self->{'toolbar'} = $self->{'mw'}->ToolBar(qw/-movable 1 -side top/ );
#					      -indicatorcolor blue/);

  $self->{'toolbar'}->ToolButton(-image   => 'fileopen22',
				 -tip     => _t('Open File'),
				 -command => sub {
				   $self->portfolio_open();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'appsheet16',
				 -tip     => _t('Sync with DepotXML'),
				 -command => sub {
				   $self->portfolio_sync_depot();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'caltoday16',
				 -tip     => _t('Sync with CSV-File'),
				 -command => sub {
				   $self->portfolio_sync_csv();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'filenew22',
				 -tip     => _t('New Trade'),
				 -command => sub {
				   $self->new_record();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'filesave22',
				 -tip     => _t('Save Trade'),
				 -command => sub {
				   $self->save_record();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'actreload22',
				 -tip     => _t('Render Text'),
				 -command => sub {
				   $self->render_text();
				 }
				);

  $self->{'toolbar'}->ToolButton(-image   => 'actredo22',
				 -tip     => _t('Export'),
				 -command => sub {
				   $self->portfolio_export_csv();
				 }
				);
}

############################################################
sub create_filter {
############################################################
  my $self = shift;

  open F, TJournal::Conf::get("Data::FilterFile") or return;
  my $last = "";
  while (<F>) {
    chomp;
    if ( $last ne "" && $_ !~ /\|\|/ ) {
      $self->{'filters'}->{$last} .= $_ . "\n";
    } else {
      my ($name, $filter) = split /\|\|/, $_, 2;
      $self->{'filters'}->{$name} = $filter;
      $last = $name;
    }
  }
  close F;

  my @filters = sort keys %{$self->{'filters'}};
  unshift @filters, "";
  my $ref = "";
  $self->{'selected_filter'} = \$ref;
  $self->{'toolbar_filter'} = $self->{'toolbar'}->Optionmenu(
							     -options  => \@filters,
							     -variable => $self->{'selected_filter'},
							     -command  => sub {
							       my $filter = ${ $self->{'selected_filter'} };
							     return if ( $filter eq "" );
							     $self->{'data'}->filter( $self->{'filters'}->{$filter} );
							     $self->update_all();
							   })->pack(qw/-side left/);

}

############################################################
sub update_toolbar_filter {
############################################################
  my $self = shift;
  my @filters = sort keys %{$self->{'filters'}};
  unshift @filters, "";
  $self->{'toolbar_filter'}->configure( -options  => \@filters );

}

############################################################
sub create_table_overview {
############################################################
  my $self = shift;
  my $row = 1;
  my $col = 8;

  ($row, $col) = $self->update_table();

  $self->{'table'} = $self->{'nb_overview'}->Scrolled('TableMatrix',
						      -justify => 'right',
						      -rows => $row,
						      -cols => $col+1,
						      -width => 6,
						      -height => $row,
						      -titlerows => 1,
						      -titlecols => 0,
						      -variable => $self->{'table-data'},
						      -roworigin =>  0,
						      -colorigin  => 0,
						      -colstretchmode => 'all',
						      -rowstretchmode => 'all',
						      -selectmode => 'browse',
						      -state => "disabled",
						      -sparsearray => 0,
						      -wrap => 1,
						      -multiline => 1,
						      -colwidth => 7,
						      -rowheight => 2,
						      -selcmd => sub {
							#print STDERR "SELECTED\n";
							#print STDERR join("\n", @_) . "\n";
							$self->{'table'}->tagDelete("sel2");
							$self->{'table'}->tagConfigure('sel2', -bg => 'orange', -relief => 'sunken');
							my @idx = $self->{'table'}->curselection();
							if ( $idx[0] =~ /(\d+),(\d+)/) {
							  #print STDERR "TAGGING: $1 \n";
							  $self->{'table'}->tagRow("sel2", $1);
							}
							return 0;
						      },
						      -rowtagcommand => sub { 
							my $row = shift;
							return "plus" if ($self->{'table-data'}->{"$row,8"} > 0);
							return "minus";
						      }
						     )->pack(-expand => 1,
							     -fill=>'both',
							     -anchor => "nw");

  $self->{'table'}->tagConfigure('plus', -fg => 'darkgreen', -relief => 'sunken');
  $self->{'table'}->tagConfigure('minus', -fg => 'darkred', -relief => 'sunken');

  $self->{'table'}->tagConfigure('active', -bg => 'gray90', -relief => 'sunken');
  $self->{'table'}->tagConfigure( 'title', -bg => 'gray85', -fg => 'black', -relief => 'sunken');

  $self->{'table'}->tagConfigure('sel', -bg => 'orange', -relief => 'sunken');

  $self->{'table'}->colWidth(0, 20);

}


############################################################
sub create_detail_page {
############################################################
  my $self = shift;
  my ($l, $e);

  # Adjuster
  $self->{'nb_details_left'} = $self->{'nb_details'}->Frame->pack( -side => "left",
								   -fill=>'y',
								   -expand => 1 );

  $self->{'nb_details_adj1'} = $self->{'nb_details'}->Adjuster( -widget => $self->{'nb_details_left'},
								-side => "left")->pack(-side=>"left",
										       -fill=>"y");
  $self->{'nb_details_right'} = $self->{'nb_details'}->Frame->pack( -side=>'bottom',
								    -fill=>'both',
								    -expand=>1);


  $self->{'nb_details_top'} = $self->{'nb_details_right'}->Frame->pack( -side => "top",
									-fill=>'x',
									-expand => 1 );

  $self->{'nb_details_adj2'} = $self->{'nb_details_right'}->Adjuster( -widget => $self->{'nb_details_top'},
								      -side => "top")->pack(-side=>"top",
											    -fill=>"x");
  $self->{'nb_details_bot'} = $self->{'nb_details_right'}->Frame->pack( -side=>'bottom',
									-fill=>'both',
									-expand=>1);


  $self->{'nb_details_left_top'} = $self->{'nb_details_left'}->Frame->pack( -side=>"top" );

  $self->{'details'}->{'name'} = $self->{'nb_details_left_top'}->Label(-text => "NAME" );
  Tk::grid($self->{'details'}->{'name'}, "-");

  $l = $self->{'nb_details_left_top'}->Label(-text => _t("Symbol") );
  $self->{'details'}->{'wkn'} = $self->{'nb_details_left_top'}->Entry(-text => "Symbol" );
  Tk::grid($l,$self->{'details'}->{'wkn'});

  $l = $self->{'nb_details_left_top'}->Label(-text => _t("Type") );
  $self->{'details'}->{'type'} = $self->{'nb_details_left_top'}->Entry(-text => "long");
  Tk::grid($l,$self->{'details'}->{'type'});

  $l = $self->{'nb_details_left_top'}->Label(-text => _t("Number") );
  $self->{'details'}->{'nb'} = $self->{'nb_details_left_top'}->Entry(-text => "0" );
  Tk::grid($l,$self->{'details'}->{'nb'});

  $l = $self->{'nb_details_left_top'}->Label(-text => _t("Open Date") );
  $self->{'details'}->{'opendate'} = $self->{'nb_details_left_top'}->Entry(-text => _t("Open Date") );
  Tk::grid($l,$self->{'details'}->{'opendate'});

  $l = $self->{'nb_details_left_top'}->Label(-text => _t("Open Price") );
  $self->{'details'}->{'openprice'} = $self->{'nb_details_left_top'}->Entry(-text => _t("Open Price") );
  Tk::grid($l,$self->{'details'}->{'openprice'});

  $l = $self->{'nb_details_left_top'}->Label(-text => _t("Close Date") );
  $self->{'details'}->{'closedate'} = $self->{'nb_details_left_top'}->Entry(-text => _t("Close Date") );
  Tk::grid($l,$self->{'details'}->{'closedate'});

  $l = $self->{'nb_details_left_top'}->Label(-text => _t("Close Price") );
  $self->{'details'}->{'closeprice'} = $self->{'nb_details_left_top'}->Entry(-text => _t("Close Price") );
  Tk::grid($l,$self->{'details'}->{'closeprice'});

  $l = $self->{'nb_details_left_top'}->Label(-text => _t("Fee") );
  $self->{'details'}->{'fee'} = $self->{'nb_details_left_top'}->Entry(-text => _t("Fee") );
  Tk::grid($l,$self->{'details'}->{'fee'});

  # Insert the categories:
  $self->{'detail_category_frame'} = $self->{'nb_details_left'}->Frame->pack( -side=>"top",
									      -anchor => "nw");
 
  my $FONT = '-*-Helvetica-Medium-R-Normal--*-140-*-*-*-*-*-*';
  $self->{'text_output'} =  $self->{'nb_details_top'}->Scrolled('ROText',
								-font       => $FONT,
								-scrollbars => 'e',
								-wrap       => 'word')->pack(-expand => 1,
											     -fill => "both");

  $self->{'text_input'} =  $self->{'nb_details_bot'}->Scrolled('Text',
							       -scrollbars => 'e',
							       -wrap       => 'word')->pack(-expand => 1,
											    -fill => "both");

}

############################################################
sub update_all {
############################################################
  my $self = shift;
  $self->update_detail_page();
  $self->update_table();
  $self->update_analysis_trade_page();
  $self->update_portfolio_trade_page();
}

############################################################
sub update_detail_page {
############################################################
  my $self = shift;

  my $id = 0;
  my @idx = $self->{'table'}->curselection();
  if ( $idx[0] =~ /(\d+),(\d+)/) {
    $id = $self->{'table-data'}->{"$1,idx"};
  }
  $self->{'curid'} = $id;

  my $trade;
  if (defined($self->{'data'}->{'data'}->{'trade'})) {
    foreach $trade ( @{$self->{'data'}->{'data'}->{'trade'}} ) {
      if ($trade->{'id'} == $id) {

	# Name
	$self->{'details'}->{'name'}->configure(-text => $self->{'db'}->get_db_name( $trade->{'wkn'} ) );

	# Wkn
	$self->{'details'}->{'wkn'}->delete(0,"end");
	$self->{'details'}->{'wkn'}->insert("end", $trade->{'wkn'});

	# Type
	$self->{'details'}->{'type'}->delete(0,"end");
	$self->{'details'}->{'type'}->insert("end", $trade->{'type'});

	# Number
	$self->{'details'}->{'nb'}->delete(0,"end");
	$self->{'details'}->{'nb'}->insert("end", $trade->{'nb'});

	# Open Date
	$self->{'details'}->{'opendate'}->delete(0,"end");
	$self->{'details'}->{'opendate'}->insert("end", $trade->{'open'}->{'date'});

	# Open Price
	$self->{'details'}->{'openprice'}->delete(0,"end");
	$self->{'details'}->{'openprice'}->insert("end", $trade->{'open'}->{'value'});

	# Close Date
	$self->{'details'}->{'closedate'}->delete(0,"end");
	$self->{'details'}->{'closedate'}->insert("end", $trade->{'close'}->{'date'});

	# Close Price
	$self->{'details'}->{'closeprice'}->delete(0,"end");
	$self->{'details'}->{'closeprice'}->insert("end", $trade->{'close'}->{'value'});

	# Fee
	$self->{'details'}->{'fee'}->delete(0,"end");
	$self->{'details'}->{'fee'}->insert("end", $trade->{'fee'}->{'value'});

	# Text
	$self->{'text_input'}->delete("1.0", "end");
	if ( ref($trade->{'text'}) =~ /ARRAY/ ) {
	  foreach my $line (@{$trade->{'text'}}) {
	    $self->{'text_input'}->insert("end", $line) 
	      unless (ref($line) =~/HASH/ );
	  }
	} else {
	  $self->{'text_input'}->insert("end", $trade->{'text'} );
	}

	# Render the Text
	$self->render_text();

	# Add the categories
		foreach ( $self->{'detail_category_frame'}->packSlaves() ) {
	  $_->packForget();
	}
	$self->{'detail_category_frame'}->Label(-text => "" )->pack();

	my $f = $self->{'detail_category_frame'}->Frame()->pack(-anchor=>"nw");
	$self->{'detail_category_entry'} = $f->Entry()->pack(-anchor=>"nw",
							     -side => "left");
	$f->Button( -text => "ADD", 
		    -command => sub {
		      foreach my $t ( @{$self->{'data'}->{'data'}->{'trade'}} ) {
			if ($t->{'id'} == $self->{'curid'}) {
			  my $ref = {};
			  $ref->{name} = $self->{'detail_category_entry'}->get();
			  return if ($ref->{name} eq "");
			  push @{$t->{'category'}}, $ref;
			}
		      }
		      $self->update_detail_page();
		    })->pack(-side => "left",
			     -anchor=>"nw");

	foreach my $cat ( sort $self->{data}->collect_categories() ) {
	  # Add a Checkbox
	  my $var = 0;
	  $self->{'category_checkbox'}->{$cat} = \$var;
	  my $cb = $self->{'detail_category_frame'}->Checkbutton( -text => $cat,
								  -variable => $self->{'category_checkbox'}->{$cat},
								  -onvalue => 1,
								  -offvalue => 0
								)->pack( -anchor => "nw");
	  $cb->configure( -command => [ sub {
					  my $c = shift;
					  my $on = ${ $self->{'category_checkbox'}->{$c} };
					  print "STATE: $on\n";
					  foreach my $t ( @{$self->{'data'}->{'data'}->{'trade'}} ) {
					    if ($t->{'id'} == $self->{'curid'}) {

					      if ( $on == 0 && defined($t->{'category'}) ) {
						foreach my $categorie ( 0..$#{$t->{'category'}} ) {
						  if ( $trade->{'category'}->[$categorie]->{name} eq $c ) {
						    splice @{$t->{'category'}}, $categorie, 1;
						  }
						}
					      } elsif ( $on == 1 && defined($t->{'category'}) ) {
						my $ref = {};
						$ref->{name} = $c;
						push @{$t->{'category'}}, $ref;
					      } elsif ( $on == 1 && !defined($t->{'category'}) ) {
						@{$t->{'category'}} = ();
						my $ref = {};
						$ref->{name} = $c;
						push @{$t->{'category'}}, $ref;
					      }
					      last;
					    }
					  }
					}, $cat ] );
	  if (defined($trade->{'category'})) {
	    foreach my $categorie ( @{$trade->{'category'}} ) {
	      if ( $categorie->{name} eq $cat ) {
		$cb->select();
	      }
	    }
	  }
	}

	last;
      }
    }
  }

}

############################################################
sub save_record {
############################################################
  my $self = shift;

  my $trade;
  if (defined($self->{'data'}->{'data'}->{'trade'})) {
    foreach $trade ( @{$self->{'data'}->{'data'}->{'trade'}} ) {
      if ($trade->{'id'} == $self->{'curid'}) {

	# Wkn
	$trade->{'wkn'} = $self->{'details'}->{'wkn'}->get();

	# Type
	$trade->{'type'} = $self->{'details'}->{'type'}->get();

	# Number
	$trade->{'nb'} = $self->{'details'}->{'nb'}->get();

	# Open Date
	$trade->{'open'}->{'date'} = $self->{'details'}->{'opendate'}->get();

	# Open Price
	$trade->{'open'}->{'value'} = $self->{'details'}->{'openprice'}->get();

	# Close Date
	$trade->{'close'}->{'date'} = $self->{'details'}->{'closedate'}->get();

	# Close Price
	$trade->{'close'}->{'value'} = $self->{'details'}->{'closeprice'}->get();

	# Fee
	$trade->{'fee'}->{'value'} = $self->{'details'}->{'fee'}->get();

	# Text
	$trade->{'text'} = [];
	$trade->{'text'}->[0] = $self->{'text_input'}->get("1.0", "end");
	$trade->{'text'}->[1] = " ";

	last;
      }
    }
  }

  $self->update_table();

}


############################################################
sub new_record {
############################################################
  my $self = shift;

  my $trade;
  $trade->{'wkn'} = "";
  $trade->{'id'} = $self->{'data'}->newid();
  $trade->{'type'} = "long";
  $trade->{'nb'} = "0";
  $trade->{'open'}->{'date'} = timestamp();
  $trade->{'open'}->{'value'} = "0";
  $trade->{'close'}->{'date'} = timestamp();
  $trade->{'close'}->{'value'} = "0";
  $trade->{'fee'}->{'value'} = "0";
  @{$trade->{'text'}} = ();
  $trade->{'text'}->[0] = _t("Comments") . ":";
  $trade->{'text'}->[1] = " ";

  push @{$self->{'data'}->{'data'}->{'trade'}}, $trade;

  $self->update_table();
  $self->{'table'}->selectionSet("1,1","1,1");
  $self->update_detail_page();

}


############################################################
sub render_text {
############################################################
  my $self = shift;
  my $content = $self->{'text_input'}->get("1.0", "end");

  $self->{'text_output'}->delete("1.0", "end");
  $self->{'text_output'}->tagConfigure(qw/head1 -font -*-Helvetica-Bold-R-Normal--*-180-*-*-*-*-*-*/);
  $self->{'text_output'}->tagConfigure(qw/head2 -font -*-Helvetica-Bold-R-Normal--*-160-*-*-*-*-*-*/);

  foreach my $line ( split /\n/s, $content ) {
    if ( $line =~ /^=(\w+)\s+(.*)/ ) {
      my $tag = $1;
      my $content = $2;
      if ($tag eq "img") {
	my $image = $self->{'mw'}->Photo( -file => $content );
	if ( $image->width > $self->{'text_output'}->width |
	     $image->height > $self->{'text_output'}->height ) {
	  my $factor = $self->{'text_output'}->width / $image->width;
	  $factor = $self->{'text_output'}->height / $image->height 
	    if ($self->{'text_output'}->height / $image->height  < $factor);
	  my $subs = $self->{'mw'}->Photo;
	  my $intfactor = int( 1 / $factor );
	  $intfactor++ if ($intfactor < (1/$factor));
	  $intfactor = 1 if ($intfactor < 1);
	  $subs->copy($image, -subsample => ($intfactor, $intfactor) );
	  $image->blank;
	  $image->copy($subs);
	  $subs->delete();
	}
	$self->{'text_output'}->imageCreate("end", -image => $image ) 
	  unless ( ! -r $content );
      } else {
	$self->{'text_output'}->insert("end", $content ."\n", $tag);
      }
    } else {
      $self->{'text_output'}->insert("end", $line ."\n");
    }
  }
}

############################################################
sub update_table {
############################################################
  my $self = shift;
  my $row = 1;
  my $col = 8;

  $self->{'table-data'}->{"0,0"} = _t("Symbol");
  $self->{'table-data'}->{"0,1"} = _t("Type");
  $self->{'table-data'}->{"0,2"} = _t("Nb");
  $self->{'table-data'}->{"0,3"} = _t("Open Date");
  $self->{'table-data'}->{"0,4"} = _t("Open Price");
  $self->{'table-data'}->{"0,5"} = _t("Close Date");
  $self->{'table-data'}->{"0,6"} = _t("Close Price");
  $self->{'table-data'}->{"0,7"} = _t("Fee");
  $self->{'table-data'}->{"0,8"} = _t("Win/Loss");

  if (defined($self->{'data'}->{'data'}->{'trade'})) {
    foreach my $trade ( sort {$b->{'open'}->{'date'} cmp $a->{'open'}->{'date'}} 
			@{$self->{'data'}->{'data'}->{'trade'}} ) {

      $self->{'table-data'}->{"$row,0"} = $trade->{'wkn'};
      $self->{'table-data'}->{"$row,1"} = $trade->{'type'};
      $self->{'table-data'}->{"$row,2"} = $trade->{'nb'};
      $self->{'table-data'}->{"$row,3"} = $trade->{'open'}->{'date'};
      $self->{'table-data'}->{"$row,3"} =~ s/_/ /;
      $self->{'table-data'}->{"$row,4"} = $trade->{'open'}->{'value'};
      $self->{'table-data'}->{"$row,5"} = $trade->{'close'}->{'date'};
      $self->{'table-data'}->{"$row,5"} =~ s/_/ /;
      $self->{'table-data'}->{"$row,6"} = $trade->{'close'}->{'value'};
      $self->{'table-data'}->{"$row,7"} = $trade->{'fee'}->{'value'};

      my $prozent;
      if ( defined( $trade->{'fee'}->{'value'} ) ) {
	$prozent = ( 100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
			     ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) -
			     $trade->{'fee'}->{'value'} ) ) / 
			       ($trade->{'nb'}*$trade->{'open'}->{'value'})
				   unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
	if ($trade->{'type'} =~ /short/i) {
	  $prozent = ( -100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
				( $trade->{'nb'} * $trade->{'open'}->{'value'} ) +
				$trade->{'fee'}->{'value'} ) ) / 
				  ($trade->{'nb'}*$trade->{'open'}->{'value'})
				    unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
	}
      }
      $self->{'table-data'}->{"$row,8"} = sprintf("%.2f",$prozent);
      $self->{'table-data'}->{"$row,8"} = "+" . $self->{'table-data'}->{"$row,8"} if ($prozent > 0);

      $self->{'table-data'}->{"$row,idx"} = $trade->{'id'};

      $row++;
    }
  }

  if ( defined( $self->{'table'} ) ) {
    $self->{'table'}->configure(-rows => $row,
				-cols => $col+1 );
  }

  return ($row, $col);
}


############################################################
sub create_analysis_trade_page {
############################################################
  my $self = shift;

  $self->{'nb_analysis_trade_top'} = 
    $self->{'nb_analysis_trade'}->Frame->pack( -side=>"top",
					     -anchor=>"nw" );

  my $l;

  $self->{'analysis_trade'}->{'name'} = $self->{'nb_analysis_trade_top'}->Label(-text => "NAME" );
  Tk::grid($self->{'analysis_trade'}->{'name'}, "-",-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Symbol") . ":" );
  $self->{'analysis_trade'}->{'wkn'} = $self->{'nb_analysis_trade_top'}->Label(-text => "WKN" );
  Tk::grid($l,$self->{'analysis_trade'}->{'wkn'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Type") . ":" );
  $self->{'analysis_trade'}->{'type'} = $self->{'nb_analysis_trade_top'}->Label(-text => "long");
  Tk::grid($l,$self->{'analysis_trade'}->{'type'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Number") . ":" );
  $self->{'analysis_trade'}->{'nb'} = $self->{'nb_analysis_trade_top'}->Label(-text => "0" );
  Tk::grid($l,$self->{'analysis_trade'}->{'nb'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Open Date") . ":" );
  $self->{'analysis_trade'}->{'opendate'} = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Open Date") );
  Tk::grid($l,$self->{'analysis_trade'}->{'opendate'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Open Price") . ":" );
  $self->{'analysis_trade'}->{'openprice'} = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Open Price") );
  Tk::grid($l,$self->{'analysis_trade'}->{'openprice'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Close Date") . ":" );
  $self->{'analysis_trade'}->{'closedate'} = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Close Date") );
  Tk::grid($l,$self->{'analysis_trade'}->{'closedate'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Close Price") . ":" );
  $self->{'analysis_trade'}->{'closeprice'} = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Close Price") );
  Tk::grid($l,$self->{'analysis_trade'}->{'closeprice'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Fee") . ":" );
  $self->{'analysis_trade'}->{'fee'} = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Fee") );
  Tk::grid($l,$self->{'analysis_trade'}->{'fee'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => "" );
  Tk::grid($l,$l);

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Percentage") . ":" );
  $self->{'analysis_trade'}->{'percentage'} = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Percentage") );
  Tk::grid($l,$self->{'analysis_trade'}->{'percentage'},-sticky=>'w');

  $l = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Net") . ":" );
  $self->{'analysis_trade'}->{'net'} = $self->{'nb_analysis_trade_top'}->Label(-text => _t("Net") );
  Tk::grid($l,$self->{'analysis_trade'}->{'net'},-sticky=>'w');



}


############################################################
sub update_analysis_trade_page {
############################################################
  my $self = shift;

  my $id = 0;
  my @idx = $self->{'table'}->curselection();
  if ( $idx[0] =~ /(\d+),(\d+)/) {
    $id = $self->{'table-data'}->{"$1,idx"};
  }
  $self->{'curid'} = $id;

  my $trade;
  if (defined($self->{'data'}->{'data'}->{'trade'})) {
    foreach $trade ( @{$self->{'data'}->{'data'}->{'trade'}} ) {
      if ($trade->{'id'} == $id) {

	# Name
	$self->{'analysis_trade'}->{'name'}->configure(-text => $self->{'db'}->get_db_name( $trade->{'wkn'} ) );

	# Wkn
	$self->{'analysis_trade'}->{'wkn'}->configure(-text => $trade->{'wkn'});

	# Type
	$self->{'analysis_trade'}->{'type'}->configure(-text => $trade->{'type'});

	# Number
	$self->{'analysis_trade'}->{'nb'}->configure(-text => $trade->{'nb'});

	# Open Date
	$self->{'analysis_trade'}->{'opendate'}->configure(-text => $trade->{'open'}->{'date'});

	# Open Price
	$self->{'analysis_trade'}->{'openprice'}->configure(-text => $trade->{'open'}->{'value'});

	# Close Date
	$self->{'analysis_trade'}->{'closedate'}->configure(-text => $trade->{'close'}->{'date'});

	# Close Price
	$self->{'analysis_trade'}->{'closeprice'}->configure(-text => $trade->{'close'}->{'value'});

	# Fee
	$self->{'analysis_trade'}->{'fee'}->configure(-text => $trade->{'fee'}->{'value'});

	# Percentage
	my $prozent;
	if ( defined( $trade->{'fee'}->{'value'} ) ) {
	  $prozent = ( 100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
			       ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) -
			       $trade->{'fee'}->{'value'} ) ) / 
				 ($trade->{'nb'}*$trade->{'open'}->{'value'})
				   unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
	  if ($trade->{'type'} =~ /short/i) {
	    $prozent = ( -100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
				  ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) +
				  $trade->{'fee'}->{'value'} ) ) / 
				    ($trade->{'nb'}*$trade->{'open'}->{'value'})
				      unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
	  }
	}

	$prozent = sprintf("%.2f",$prozent);
	$prozent = "+" . $prozent if ($prozent > 0);
	$prozent .= "%";
	$self->{'analysis_trade'}->{'percentage'}->configure(-text => $prozent );
	
	# Net Gain / Loss:
	my $net;
	if ( defined( $trade->{'fee'}->{'value'} ) ) {
	  $net =  ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
	    ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) -
	      $trade->{'fee'}->{'value'};
	  if ($trade->{'type'} =~ /short/i) {
	    $net = ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
	      ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) +
		$trade->{'fee'}->{'value'};
	  }
	}
	$net = sprintf("%.2f",$net);
	$net = "+" . $net if ($net > 0);
	$self->{'analysis_trade'}->{'net'}->configure(-text => $net );

	last;
      }
    }
  }

}


############################################################
sub create_analysis_portfolio_page {
############################################################
  my $self = shift;

  $self->{'nb_analysis_portfolio_top'} = 
    $self->{'nb_analysis_portfolio'}->Scrolled('Pane',
					       -scrollbars => "e" )->pack( -fill => "both",
									   -expand => 1,
									   -side=>"top",
									   -anchor=>"nw" );
  $self->create_pf_analysis();

}

############################################################
sub create_pf_analysis {
############################################################
  my $self = shift;

  TJournal::Conf::default("PFAnalysis::Color::neutral", $self->{'nb_analysis_portfolio_top'}->cget(-background ));
  my @modules = split /\|/, TJournal::Conf::get("PFAnalysis::Display");

  foreach my $module ( @modules ) {

    my $ana;
    my $eval = "use TJournal::PFAnalysis::$module;\n" .
      "\$ana = TJournal::PFAnalysis::$module->new ( \$self->{'data'}->{'data'} );";
    eval $eval;
    if (!defined($ana)) {
      warn "Module $module not found!:\n" . $@;
      next;
    }

    my $f = $self->{'nb_analysis_portfolio_top'}->Frame( -relief => 'groove',
							 -borderwidth => 5 )->pack( -fill => 'x',
										    -expand => 1,
										    -anchor => "nw" );
    my $sf1 = $f->Frame()->pack(-fill => 'x',
				-expand => 1,
				-side => "left" );
    my $sf2 = $f->Frame( -width => 100,
			 -height => 50 )->pack( -side => "right" );

    my $l1 = $sf1->Label( -text => $ana->name_t(),
			  -font => [ -family => "Arial",
				     -size => 12,
				     -weight => "bold" ])->pack( -anchor => "nw" );

    my $t = $sf1->Scrolled("ROText",
			   -font => [ -family => "Arial" ],
			   -wrap => "word",
			   -height => 3,
			   -background => $sf1->cget(-background ),
			   -relief => 'flat',
			   -borderwidth => 0,
			   -scrollbars => "e" )->pack(-anchor => "nw",
						      -fill => "x" );
    $t->insert('0.end', $ana->info_t());

    my $col = TJournal::Conf::get("PFAnalysis::Color::" . $ana->color() );
    $col = $sf1->cget(-background ) unless ( defined($col) );
    $sf2->Label( -text => $ana->formated(),
		 -width => 13,
		 -height => 3,
		 -background => $col )->pack( -side => "top" );
  }
}


############################################################
sub update_portfolio_trade_page {
############################################################
  my $self = shift;
  $self->{'nb_analysis_portfolio_top'}->packForget();
  $self->create_analysis_portfolio_page();
}



############################################################
sub timestamp { # Creates a timestamp
############################################################
  my @t = localtime(time);
  $t[5] += 1900;
  $t[4] += 1;
  my $date = sprintf("%04d-%02d-%02d_%02d:%02d:%02d", $t[5], $t[4], $t[3], $t[2], $t[1], $t[0]);
  return $date;
}

############################################################
sub portfolio_new {
############################################################
  my $self = shift;
  my @types =
    (["XML Datei",           [qw/.xml/]],
     ["All files",                '*']
    );
  my $file = $self->{'mw'}->getSaveFile(-filetypes => \@types,
					-initialdir => $ENV{PWD},
					-initialfile => 'data.xml');
  $self->{'data'} = TJournal::Data->new( $file );
  $self->{'data'}->writefile();

  $self->update_table();
  $self->update_detail_page();
}

############################################################
sub portfolio_open {
############################################################
  my $self = shift;
  my @types =
    (["XML Datei",           [qw/.xml/]],
     ["All files",                '*']
    );
  my $file = $self->{'mw'}->getOpenFile(-filetypes => \@types,
					-initialdir => $ENV{PWD},
					-initialfile => 'data.xml');
  $self->{'data'}->readfile( $file );
  $self->update_table();
  $self->{'table'}->selectionSet("1,1","1,1");
  $self->update_detail_page();
}

############################################################
sub portfolio_save_as {
############################################################
  my $self = shift;
  my @types =
    (["XML Datei",           [qw/.xml/]],
     ["All files",                '*']
    );
  my $file = $self->{'mw'}->getSaveFile(-filetypes => \@types,
					-initialdir => $ENV{PWD},
					-initialfile => 'data.xml');
  $self->{'data'}->{file} =  $file;
  $self->{'data'}->writefile();

  $self->update_table();
  $self->update_detail_page();
}

############################################################
sub portfolio_export_csv {
############################################################
  my $self = shift;
  my @types =
    ([_t("Txt files"),           [qw/.txt/]],
     [_t("All files"),                '*']
    );
  my $file = $self->{'mw'}->getSaveFile(-filetypes => \@types,
					-initialdir => $ENV{'PWD'},
					-initialfile => 'Untitled',
					-defaultextension => '.txt');
  my ($row, $col) = $self->update_table();
  open OUT, ">$file" or die "Could not write to $file...";
  if (defined($self->{'data'}->{'data'}->{'trade'})) {
    print OUT "# " . join("\t", ("id", "code", "nb", "type", "opendate", 
				 "openvalue", "closedate", "closevalue",
				 "fee", "text") ) . "\n";
    foreach my $pos ( @{$self->{'data'}->{'data'}->{'trade'}} ) {
      print OUT $pos->{'id'} . "\t";
      print OUT $pos->{'wkn'} . "\t";
      print OUT $pos->{'nb'} . "\t";
      print OUT $pos->{'type'} . "\t";
      print OUT $pos->{'open'}->{'date'} . "\t";
      print OUT $pos->{'open'}->{'value'} . "\t";
      print OUT $pos->{'close'}->{'date'} . "\t";
      print OUT $pos->{'close'}->{'value'} . "\t";
      print OUT $pos->{'fee'}->{'value'} . "\t";
      if ( ref($pos->{'text'}) =~ /ARRAY/ ) {
	foreach my $line (@{$pos->{'text'}}) {
	  my $l = $line;
	  $l =~ s/^/ /mg;
	  print OUT $l;
	}
      } else {
	my $l = $pos->{'text'};
	$l =~ s/^/ /mg;
	print OUT $l;
      }
      print OUT "\n";
    }
  }
  close OUT;
}

############################################################
sub portfolio_sync_depot {
############################################################
  my $self = shift;

  my @types =
    ([_t("XML Files"),           [qw/.xml/]],
     [_t("All files"),                '*']
    );
  my $file = $self->{'mw'}->getOpenFile(-filetypes => \@types,
					-initialdir => $ENV{'PWD'},
					-initialfile => 'depot-000001.xml');
  my $dir = "./";
  if ( $file =~ /(.*\/)([^\/]+$)/ ) {
    $dir = $1;
    $file = $2;
  }

  require Depot::Conf;
  Depot::Conf::load();
  Depot::Conf::default("Config::Basedir", $dir);

  $self->{'data'}->syncwithdepot( $file );

  $self->update_table();
  $self->{'table'}->selectionSet("1,1","1,1");
  $self->update_detail_page();

}

############################################################
sub portfolio_sync_csv {
############################################################
  my $self = shift;
  my @types =
    ([_t("CSV files"),           [qw/.csv/]],
     [_t("Txt files"),           [qw/.txt/]],
     [_t("All files"),                '*']
    );
  my $file = $self->{'mw'}->getOpenFile(-filetypes => \@types,
					-initialdir => $ENV{'PWD'},
					-initialfile => 'import.txt');

  $self->{'data'}->syncwithtxt( $file );
  $self->update_table();
  $self->{'table'}->selectionSet("1,1","1,1");
  $self->update_detail_page();
}

############################################################
sub configure_main {
############################################################
  my $self = shift;

  if (! Exists( $self->{'win_gconf'} ) ) {
    $self->{'win_gconf'} = $self->{'mw'}->Toplevel( -title => _t("General Configuration") );
    my ($l, $b);

    # Language
    $l = $self->{'win_gconf'}->Label( -text => "Language File:" );
    my $var_lang = TJournal::Conf::get("LANG");
    $self->{'win_gconf_lang'} = $self->{'win_gconf'}->Entry( -textvariable => \$var_lang );
    $b = $self->{'win_gconf'}->Button( -text => "Browse",
				       -command => sub {
					         my $types = [
							      ['All Files',        '*',             ],
							     ];
						 my $filename = $self->{'win_gconf'}->getOpenFile(-filetypes=>$types,
												  -initialfile => $var_lang );
						 if ($filename ne "") {
						   $var_lang = "$filename";
						 }
					       });
    Tk::grid($l, $self->{'win_gconf_lang'}, $b);

    # Base directory
    $l = $self->{'win_gconf'}->Label( -text => "Base directory:" );
    my $var_base = TJournal::Conf::get("Data::Basedir");
    $self->{'win_gconf_base'} = $self->{'win_gconf'}->Entry( -textvariable => $self->{'var_base'} = \$var_base );
    $b = $self->{'win_gconf'}->Button( -text => "Browse", 
				       -command => sub {
					 my $fs = $self->{mw}->FileSelect( -verify => ['-d'] );
					 my $file = $fs->Show();
					 if ( $file ) {
					   $var_base = "$file";
					 }
				       });

    Tk::grid($l, $self->{'win_gconf_base'}, $b);

    # Shares file
    $l = $self->{'win_gconf'}->Label( -text => "Shares file:" );
    my $var_shares = TJournal::Conf::get("SharesFile");
    $self->{'win_gconf_shares'} = $self->{'win_gconf'}->Entry( -textvariable => $self->{'var_shares'} = \$var_shares );
    $b = $self->{'win_gconf'}->Button( -text => "Browse",
				       -command => sub {
					         my $types = [
							      ['All Files',        '*',             ],
							     ];
						 my $filename = $self->{'win_gconf'}->getOpenFile(-filetypes=>$types,
												  -initialfile => $var_shares );
						 if ($filename ne "") {
						   $var_shares = "$filename";
						 }
					       } );
    Tk::grid($l, $self->{'win_gconf_shares'}, $b);

    # Filter file
    $l = $self->{'win_gconf'}->Label( -text => "Filter file:" );
    my $var_filter = TJournal::Conf::get("Data::FilterFile");
    $self->{'win_gconf_filter'} = $self->{'win_gconf'}->Entry( -textvariable => $self->{'var_shares'} = \$var_filter );
    $b = $self->{'win_gconf'}->Button( -text => "Browse",
				       -command => sub {
					         my $types = [
							      ['All Files',        '*',             ],
							     ];
						 my $filename = $self->{'win_gconf'}->getOpenFile(-filetypes=>$types,
												  -initialfile => $var_shares );
						 if ($filename ne "") {
						   $var_filter = "$filename";
						 }
					       } );
    Tk::grid($l, $self->{'win_gconf_filter'}, $b);

    # Save or Cancel
    $b = $self->{'win_gconf'}->Button( -text => "Save",
					   -command => sub {
					     TJournal::Conf::set("LANG", $var_lang);
					     TJournal::Conf::set("SharesFile", $var_shares);
					     TJournal::Conf::set("Data::Basedir", $var_base);
					     TJournal::Conf::set("Data::FilterFile", $var_filter);
					     TJournal::Conf::save();
					     $self->{'win_gconf'}->withdraw();
					   } );
    my $b2 = $self->{'win_gconf'}->Button( -text => "Cancel",
					   -command => sub {
					     $self->{'win_gconf'}->withdraw();
					   });
    Tk::grid($b, $b2, '-');

  } else {
    $self->{'win_gconf'}->deiconify();
    $self->{'win_gconf'}->raise();
  }

}

############################################################
sub configure_filter {
############################################################
  my $self = shift;

  if (! Exists( $self->{'filter_gconf'} ) ) {
    $self->{'filter_gconf'} = $self->{'mw'}->Toplevel( -title => _t("Filter Configuration") );
    $self->{'filter_gconf_lb'} = 
      $self->{'filter_gconf'}->Scrolled('BrowseListbox',
					-browsecmd => sub {
					  my $filter = shift;
					  $filter = $self->{'filter_gconf_lb'}->get( $filter );
					  $self->{'filter_gconf_actfilter'} = $filter;
					  $self->{'filter_gconf_advanced_filterentry'}->configure(-textvariable => \$filter );
					  $self->{'filter_gconf_advanced_text'}->delete("0.0", "end");
					  $self->{'filter_gconf_advanced_text'}->insert("end", 
											$self->{'filters'}->{$filter});
					},
					-scrollbars => "se" )->pack(-side => "left",
								    -expand => 1,
								    -fill => "both");
    $self->{'filter_gconf'}->Adjuster( -widget => $self->{'filter_gconf_lb'},
				       -side => "left")->pack(-side=>"left",
							      -fill=>"y");
    $self->{'filter_gconf_notebook'} = $self->{'filter_gconf'}->NoteBook()->pack( -side => "left",
										  -expand => 1,
										  -fill => 'both' );
    $self->{'filter_gconf_simple'} = $self->{'filter_gconf_notebook'}->add("simple", 
									   -label => _t("Simple") );

    $self->{'filter_gconf_simple'}->Label( -text => "Sorry, but at the moment, it is only\n" .
					   "possible to edit the filters in advanced mode!\n\n ".
					   "Saving will be supported in the next version..." )->pack();

    $self->{'filter_gconf_advanced'} = $self->{'filter_gconf_notebook'}->add("advanced", 
									     -label => _t("Advanced") );
    my $f = $self->{'filter_gconf_advanced'}->Frame()->pack( -fill => "x",
							     -expand => 1,
							     -side => "top" );
    $f->Label( -text => "Name: ")->pack( -side => "left" );
    $self->{'filter_gconf_advanced_filterentry'} = $f->Entry(-width=>40)->pack( -side => "left",
										-fill => "x",
										-expand => 1 );
    $f->Button( -text => "Rename",
		-command => sub {
		  my $newnameref = $self->{'filter_gconf_advanced_filterentry'}->cget(-textvariable);
		  my $oldname = $self->{'filter_gconf_actfilter'};
		  my $newname = $$newnameref;
		  $self->{'filters'}->{$newname} = 
		    join("\n", $self->{'filter_gconf_advanced_text'}->get("0.0", "end"));
		  delete( $self->{'filters'}->{$oldname} );
		  $self->configure_filter();
		  $self->update_toolbar_filter();
		})->pack( -side => "right" );
    $f->Button( -text => "New",
		-command => sub {
		  my $newnameref = $self->{'filter_gconf_advanced_filterentry'}->cget(-textvariable);
		  my $newname = $$newnameref;
		  $self->{'filters'}->{$newname} = 
		    join("\n", $self->{'filter_gconf_advanced_text'}->get("0.0", "end"));
		  $self->configure_filter();
		  $self->update_toolbar_filter();
		})->pack( -side => "right" );

    # Text Window
    $self->{'filter_gconf_advanced_text'} = 
      $self->{'filter_gconf_advanced'}->Scrolled("Text",
						 -scrollbars => "se")->pack( -fill => "both",
									     -expand => 1,
									     -side => "top" );
    # Save it
    my $f1 = $self->{'filter_gconf_advanced'}->Frame()->pack( -fill => "x",
								-expand => 1,
								-side => "top" );
    $f1->Button(-text=>"Set",
		-command => sub {
		  $self->{'filters'}->{$self->{'filter_gconf_actfilter'}} =
		    join("\n", $self->{'filter_gconf_advanced_text'}->get("0.0", "end"));
		})->pack( -side => "left",
			  -fill => "y",
			  -expand => 1
			);
    $f1->Button(-text=>"Save",
		-command => sub {
		  $self->{'filters'}->{$self->{'filter_gconf_actfilter'}} =
		    join("\n", $self->{'filter_gconf_advanced_text'}->get("0.0", "end"));

		  # Save
		  open F, ">" . TJournal::Conf::get("Data::FilterFile") or return;
		  foreach my $filter ( keys %{$self->{'filters'}} ) {
		    print F $filter . "||" . $self->{'filters'}->{$filter} . "\n";
		  }
		  close F;
		})->pack( -side => "left",
			  -fill => "y",
			  -expand => 1
			);
    $f1->Button(-text=>"Close",
		-command => sub {
		  $self->{'filter_gconf'}->withdraw();
		})->pack( -side => "left",
			  -fill => "y",
			  -expand => 1
			);
  } else {
    $self->{'filter_gconf'}->deiconify();
    $self->{'filter_gconf'}->raise();
  }

  # Fill the listbox
  my @filters = sort keys %{$self->{'filters'}};
  $self->{'filter_gconf_lb'}->delete("0", "end");
  foreach my $f ( @filters ) {
    $self->{'filter_gconf_lb'}->insert("end", $f);
  }

}

############################################################
sub configure_portfolio {
############################################################
  my $self = shift;

  if (! Exists( $self->{'portfolio_gconf'} ) ) {
    # Create the Notebook and the Toplevel Window
    $self->{'portfolio_gconf'} = $self->{'mw'}->Toplevel( -title => _t("Portfolio Configuration") );
    $self->{'portfolio_gconf_notebook'} = $self->{'portfolio_gconf'}->NoteBook()->pack( -side => "left",
											-expand => 1,
											-fill => 'both' );
    $self->{'portfolio_gconf_color'} = $self->{'portfolio_gconf_notebook'}->add("color", 
										-label => _t("Colors") );
    $self->{'portfolio_gconf_grading'} = $self->{'portfolio_gconf_notebook'}->add("grading", 
										  -label => _t("Grading") );
    $self->{'portfolio_gconf_active'} = $self->{'portfolio_gconf_notebook'}->add("active", 
										 -label => _t("Active Modules") );

    # Set up the 'color'-Page
    $self->{'portfolio_gconf_color_lb'} = 
      $self->{'portfolio_gconf_color'}->Scrolled('BrowseListbox',
						 -selectmode => "single",
						 -browsecmd => sub {
						   my $color = shift;
						   $color = $self->{'portfolio_gconf_color_lb'}->get( $color );
						   $self->{'portfolio_gconf_color_example'}->configure( -background => $self->{'pfcolors'}->{$color} );
						   $self->{'pfcolor_act'} = $color;
						   $self->{'pfcolor_act2'} = \$color;
						   $self->{'portfolio_gconf_color_name'}->configure( -textvariable => \$color );
						 },
						 -scrollbars => "se" )->pack(-side => "left",
									     -expand => 1,
									     -fill => "both");
    $self->{'portfolio_gconf_color'}->Adjuster( -widget => $self->{'portfolio_gconf_color_lb'},
						-side => "left")->pack(-side=>"left",
								       -fill=>"y");
    $self->{'portfolio_gconf_color_frame'} = $self->{'portfolio_gconf_color'}->Frame()->pack( -side => "left",
											      -expand => 1,
											      -fill => 'both' );
    # The buttons for configuring the Color
    # #####################################
    $self->{'portfolio_gconf_color_example'} = 
      $self->{'portfolio_gconf_color_frame'}->Button(-text=>"Set",
						     -command => sub {
						       my $name = $self->{'pfcolor_act2'};
						       my $oldcolor = $self->{'pfcolors'}->{$$name};
						       $oldcolor = "gray" if (!defined($oldcolor));
						       my $color = $self->{'portfolio_gconf_color'}->chooseColor( -initialcolor => $oldcolor );
						       TJournal::Conf::set("PFAnalysis::Color::".$$name, $color);
						       $self->{'portfolio_gconf_color_example'}->configure( -background => $color );
						     })->pack( -side => "top",
							       -expand => 1,
							       -anchor => "nw",
							       -fill => 'x' );
    $self->{'portfolio_gconf_color_name'} = $self->{'portfolio_gconf_color_frame'}->Entry()->pack( -side => "top",
												   -expand => 1,
												   -anchor => "nw",
												   -fill => 'x' );
    $self->{'portfolio_gconf_color_frame'}->Button(-text=>"Rename",
						   -command => sub {
						     my $oldname = $self->{'pfcolor_act'};
						     my $refname = $self->{'pfcolor_act2'};
						     my $newname = $$refname;
						     my $color   = $self->{'pfcolors'}->{$oldname};
						     TJournal::Conf::set("PFAnalysis::Color::$newname", $color);
						     TJournal::Conf::del("PFAnalysis::Color::$oldname");
						     $self->configure_portfolio();
						   })->pack( -side => "top",
							     -expand => 1,
							     -anchor => "nw",
							     -fill => 'x' );
    $self->{'portfolio_gconf_color_frame'}->Button(-text=>"ADD",
						   -command => sub {
						     my $color = $self->{'nb_analysis_portfolio_top'}->cget(-background );
						     my $name = "newcolor";
						     $self->{'pfcolor_act'} = $name;
						     $self->{'pfcolor_act2'} = \$name;
						     $self->{'portfolio_gconf_color_name'}->configure( -textvariable => \$name );
						     TJournal::Conf::set("PFAnalysis::Color::newcolor", $color);
						     $self->configure_portfolio();
						   })->pack( -side => "top",
							     -expand => 1,
							     -anchor => "nw",
							     -fill => 'x' );
    $self->{'portfolio_gconf_color_frame'}->Button(-text=>"Save",
						   -command => sub {
						     TJournal::Conf::save();
						   })->pack( -side => "top",
							     -expand => 1,
							     -anchor => "nw",
							     -fill => 'x' );
    $self->{'portfolio_gconf_color_frame'}->Button(-text=>"Close",
						   -command => sub {
						     $self->{'portfolio_gconf'}->withdraw();
						   })->pack( -side => "top",
							     -expand => 1,
							     -anchor => "nw",
							     -fill => 'x' );

    # Configuration of the Grading Screen
    $self->{'portfolio_gconf_grading_lb'} = 
      $self->{'portfolio_gconf_grading'}->Scrolled('BrowseListbox',
						   -selectmode => "single",
						   -exportselection => 0,
						   -browsecmd => sub {
						     $self->{'portfolio_gconf_grading_lbpos1'} = shift;
						   },
						   -scrollbars => "se" )->pack(-side => "left",
									       -expand => 1,
									       -fill => "both");
    $self->{'portfolio_gconf_grading'}->Adjuster( -widget => $self->{'portfolio_gconf_grading_lb'},
						  -side => "left")->pack(-side=>"left",
									 -fill=>"y");
    $self->{'portfolio_gconf_grading_lb2'} = 
      $self->{'portfolio_gconf_grading'}->Scrolled('BrowseListbox',
						   -selectmode => "single",
						   -exportselection => 0,
						   -browsecmd => sub {
						     $self->{'portfolio_gconf_grading_lbpos2'} = shift;
						     my $modname = $self->{'portfolio_gconf_grading_lb'}->get( $self->{'portfolio_gconf_grading_lbpos1'} );
						     my $colname = $self->{'portfolio_gconf_grading_lb2'}->get( $self->{'portfolio_gconf_grading_lbpos2'} );
						     my $confstring = TJournal::Conf::get( "pfanalysis::".$modname."::coloring" );
						     my @grads = split/;/, $confstring;
						     my %hash = {};
						     foreach my $g ( @grads ) {
						       if ( $g =~ /^(.*?):(.*?)~(.*?)$/ ) {
							 $hash{lc($1)}{from} = $2;
							 $hash{lc($1)}{to} = $3;
						       }
						     }
						     my $from = $hash{lc($colname)}{from};
						     my $to = $hash{lc($colname)}{to};
						     $self->{'portfolio_gconf_grading_from'} = \$from;
						     $self->{'portfolio_gconf_grading_to'} = \$to;
						     $self->{'portfolio_gconf_grading_fromentry'}->configure( -textvariable => $self->{'portfolio_gconf_grading_from'} );
						     $self->{'portfolio_gconf_grading_toentry'}->configure( -textvariable => $self->{'portfolio_gconf_grading_to'} );
						   },
						   -scrollbars => "se" )->pack(-side => "left",
									       -expand => 1,
									       -fill => "both");
    $self->{'portfolio_gconf_grading'}->Adjuster( -widget => $self->{'portfolio_gconf_grading_lb2'},
						  -side => "left")->pack(-side=>"left",
									 -fill=>"y");
    $self->{'portfolio_gconf_grading_frame'} = $self->{'portfolio_gconf_grading'}->Frame()->pack( -side => "left",
												  -expand => 1,
												  -fill => 'both' );
    my $f1 = $self->{'portfolio_gconf_grading_frame'}->Frame()->pack( -side => "top",
								      -expand => 1,
								      -anchor => "nw",
								      -fill => 'x' );
    $f1->Label(-text => "From: ")->pack( -side => "left" );
    $self->{'portfolio_gconf_grading_fromentry'} = $f1->Entry()->pack( -side => "left" );

    my $f2 = $self->{'portfolio_gconf_grading_frame'}->Frame()->pack( -side => "top",
								      -expand => 1,
								      -anchor => "nw",
								      -fill => 'x' );
    $f2->Label(-text => "To: ")->pack( -side => "left" );
    $self->{'portfolio_gconf_grading_toentry'} = $f2->Entry()->pack( -side => "left" );

    $self->{'portfolio_gconf_grading_frame'}->Button(-text=>"Set",
						     -command => sub {
						       my $from = ${$self->{'portfolio_gconf_grading_from'}};
						       my $to = ${$self->{'portfolio_gconf_grading_to'}};
                                                       print "$from / $to \n";
                                                       my $modname = $self->{'portfolio_gconf_grading_lb'}->get( $self->{'portfolio_gconf_grading_lbpos1'} );
						       my $colname = $self->{'portfolio_gconf_grading_lb2'}->get( $self->{'portfolio_gconf_grading_lbpos2'} );
						       my $confstring = TJournal::Conf::get( "pfanalysis::".$modname."::coloring" );
						       my @grads = split/;/, $confstring;
						       my %hash = {};
						       foreach my $g ( @grads ) {
						         if ( $g =~ /^(.*?):(.*?)~(.*?)$/ ) {
							   $hash{lc($1)}{from} = $2;
							   $hash{lc($1)}{to} = $3;
						         }
						       }
                                                       delete( $hash{lc($colname)} ) if ( $to eq "" || $from eq "" );

                                                       # Set the result
						       foreach my $g ( @grads ) {
						         if ( $g =~ /^(.*?):(.*?)~(.*?)$/ ) {
							   $g = lc($1).":$from" . "~" . $to if ($1 eq lc($colname));
						         }
						       }
                                                       TJournal::Conf::set( "pfanalysis::".$modname."::coloring",
									    join(";", @grads) );
						    })->pack( -side => "top",
							      -expand => 1,
							      -anchor => "nw",
							      -fill => 'x' );
    $self->{'portfolio_gconf_grading_frame'}->Button(-text=>"Save",
						     -command => sub {
						       TJournal::Conf::save();
						     })->pack( -side => "top",
							       -expand => 1,
							       -anchor => "nw",
							       -fill => 'x' );
    $self->{'portfolio_gconf_grading_frame'}->Button(-text=>"Close",
						     -command => sub {
						       $self->{'portfolio_gconf'}->withdraw();
						     })->pack( -side => "top",
							       -expand => 1,
							       -anchor => "nw",
							       -fill => 'x' );



    # Configure the active modules
    $self->{'portfolio_gconf_active_lb'} = 
      $self->{'portfolio_gconf_active'}->Scrolled('BrowseListbox',
						  -browsecmd => sub {
						    $self->{'portfolio_gconf_active_lbpos1'} = shift;
						  },
						  -scrollbars => "se" )->pack(-side => "left",
									      -expand => 1,
									      -fill => "both");
    $self->{'portfolio_gconf_active'}->Adjuster( -widget => $self->{'portfolio_gconf_active_lb'},
						 -side => "left")->pack(-side=>"left",
									-fill=>"y");
    $self->{'portfolio_gconf_active_frame'} = $self->{'portfolio_gconf_active'}->Frame()->pack( -side => "left",
												-expand => 1,
												-fill => 'both' );
    $self->{'portfolio_gconf_active'}->Adjuster( -widget => $self->{'portfolio_gconf_active_frame'},
						 -side => "left")->pack(-side=>"left",
									-fill=>"y");
    $self->{'portfolio_gconf_active_lb2'} = 
      $self->{'portfolio_gconf_active'}->Scrolled('BrowseListbox',
						  -browsecmd => sub {
						    $self->{'portfolio_gconf_active_lbpos2'} = shift;
						  },
						  -scrollbars => "se" )->pack(-side => "left",
									      -expand => 1,
									      -fill => "both");
    $self->{'portfolio_gconf_active_frame'}->Button( -text => " << Add module",
						     -command => sub {
						       my @modules =  split /\|/, TJournal::Conf::get("PFAnalysis::Display");
						       my @modules2 =  split /\|/, TJournal::Conf::get("PFAnalysis::DisplayAll");
						       my $pos = $self->{'portfolio_gconf_active_lbpos1'};
						       my $pos2 = $self->{'portfolio_gconf_active_lbpos2'};
						       push @modules, $modules2[$pos2];
						       TJournal::Conf::set("PFAnalysis::Display", join("|", @modules));
						       $self->configure_portfolio();
						     } )->pack( -side => "top",
								-fill => "x",
								-expand => 1);
    $self->{'portfolio_gconf_active_frame'}->Button( -text => " ^ Move Up",
						     -command => sub {
						       my @modules =  split /\|/, TJournal::Conf::get("PFAnalysis::Display");
						       my $pos = $self->{'portfolio_gconf_active_lbpos1'};
						       return if ($pos==0);
						       ($modules[$pos], $modules[$pos-1]) = ($modules[$pos-1], $modules[$pos]);
						       TJournal::Conf::set("PFAnalysis::Display", join("|", @modules));
						       $self->configure_portfolio();
						     })->pack( -side => "top",
							       -fill => "x",
							       -expand => 1);
    $self->{'portfolio_gconf_active_frame'}->Button( -text => " v Move Down",
						     -command => sub {
						       my @modules =  split /\|/, TJournal::Conf::get("PFAnalysis::Display");
						       my $pos = $self->{'portfolio_gconf_active_lbpos1'};
						       return if ($pos==$#modules);
						       ($modules[$pos], $modules[$pos+1]) = ($modules[$pos+1], $modules[$pos]);
						       TJournal::Conf::set("PFAnalysis::Display", join("|", @modules));
						       $self->configure_portfolio();
						     } )->pack( -side => "top",
								-fill => "x",
								-expand => 1);
    $self->{'portfolio_gconf_active_frame'}->Button( -text => "Delete",
						     -command => sub {
						       my @modules =  split /\|/, TJournal::Conf::get("PFAnalysis::Display");
						       my $pos = $self->{'portfolio_gconf_active_lbpos1'};
						       splice @modules, $pos, 1;
						       TJournal::Conf::set("PFAnalysis::Display", join("|", @modules));
						       $self->configure_portfolio();
						     } )->pack( -side => "top",
										-fill => "x",
										-expand => 1);
    $self->{'portfolio_gconf_active_frame'}->Button(-text=>"Save",
						    -command => sub {
						      TJournal::Conf::save();
						    })->pack( -side => "top",
							      -expand => 1,
							      -anchor => "nw",
							      -fill => 'x' );
    $self->{'portfolio_gconf_active_frame'}->Button(-text=>"Close",
						    -command => sub {
						      $self->{'portfolio_gconf'}->withdraw();
						    })->pack( -side => "top",
							      -expand => 1,
							      -anchor => "nw",
							      -fill => 'x' );

  } else {
    $self->{'portfolio_gconf'}->deiconify();
    $self->{'portfolio_gconf'}->raise();
  }

  # Fill in the actual colors
  $self->{'portfolio_gconf_color_lb'}->delete("0", "end");
  $self->{'pfcolors'} = {};
  foreach my $name ( keys %TJournal::Conf::depconf ) {
    if ( $name =~ /PFAnalysis::Color::(.*)$/i ) {
      $self->{'portfolio_gconf_color_lb'}->insert("end", $1);
      $self->{'pfcolors'}->{$1} = TJournal::Conf::get("PFAnalysis::Color::".$1);
    }
  }

  # Insert all modules in grading
  $self->{'portfolio_gconf_grading_lb'}->delete("0", "end");
  my @modules =  split /\|/, TJournal::Conf::get("PFAnalysis::DisplayAll");
  foreach my $name ( @modules ) {
    $self->{'portfolio_gconf_grading_lb'}->insert("end", $name);
  }
  # ...and the colors
  $self->{'portfolio_gconf_grading_lb2'}->delete("0", "end");
  foreach my $name ( keys %TJournal::Conf::depconf ) {
    if ( $name =~ /PFAnalysis::Color::(.*)$/i ) {
      $self->{'portfolio_gconf_grading_lb2'}->insert("end", $1);
    }
  }

  # Insert the actual modules
  $self->{'portfolio_gconf_active_lb'}->delete("0", "end");
  @modules =  split /\|/, TJournal::Conf::get("PFAnalysis::Display");
  foreach my $name ( @modules ) {
    $self->{'portfolio_gconf_active_lb'}->insert("end", $name);
  }
  # Insert all available modules
  # A dynamic solution would be bette but this would requite with File::Find an other package...
  $self->{'portfolio_gconf_active_lb2'}->delete("0", "end");
  @modules =  split /\|/, TJournal::Conf::get("PFAnalysis::DisplayAll");
  foreach my $name ( @modules ) {
    $self->{'portfolio_gconf_active_lb2'}->insert("end", $name);
  }

}


1;
