package TJournal::PFAnalysis::NbGains;

# PFAnalysis.pm -- author: Oliver Bossert
# Copyright (C) 2004 Oliver Bossert
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;

use TJournal::Conf;
use TJournal::DB;

use TJournal::PFAnalysis;
our @ISA = qw(TJournal::PFAnalysis);


############################################################
sub calc {
############################################################
  my $self = shift;

  return "" if (!defined($self->{'data'}->{'trade'}));

  my $nbwin = 0;
  my $nbloss = 0;
  my $trade;

  foreach $trade ( @{$self->{'data'}->{'trade'}} ) {

    # Percentage
    my $prozent;
    if ( defined( $trade->{'fee'}->{'value'} ) ) {
      $prozent = ( 100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
			   ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) -
			   $trade->{'fee'}->{'value'} ) ) / 
			     ($trade->{'nb'}*$trade->{'open'}->{'value'})
			       unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
      if ($trade->{'type'} =~ /short/i) {
	$prozent = ( -100 * ( ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
			      ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) +
			      $trade->{'fee'}->{'value'} ) ) / 
				($trade->{'nb'}*$trade->{'open'}->{'value'})
				  unless( ($trade->{'nb'}*$trade->{'open'}->{'value'})==0);
      }
    }

    # Net Gain / Loss:
    my $net;
    if ( defined( $trade->{'fee'}->{'value'} ) ) {
      $net =  ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
	( $trade->{'nb'} * $trade->{'open'}->{'value'} ) -
	  $trade->{'fee'}->{'value'};
      if ($trade->{'type'} =~ /short/i) {
	$net = ( $trade->{'nb'} * $trade->{'close'}->{'value'} ) - 
	  ( $trade->{'nb'} * $trade->{'open'}->{'value'} ) +
	    $trade->{'fee'}->{'value'};
      }
    }

    $nbwin++ if ( $prozent >= 0);
    $nbloss++ if ( $prozent < 0);

  }

  return $nbwin;
}


############################################################
sub color {
############################################################
  my $self = shift;

  my $cal = $self->calc();
  TJournal::Conf::default("PFAnalysis::NbGains::Coloring", 
			  "verybad:0~1");
  my $line = TJournal::Conf::get("PFAnalysis::NbGains::Coloring");
  my $col = 'neutral';
  foreach my $val ( split /;/, $line ) {
    if ( $val =~ /(\w+):([\d\.-]+)~([\d\.-]+)/ ) {
      if ( $cal >= $2 && $cal < $3 ) {
	$col = $1;
      }
    }
  }
  return $col;
}

############################################################
sub name {
############################################################
  my $self = shift;
  return "Number of winning trades";
}

############################################################
sub info {
############################################################
  my $self = shift;
  return "The number of trades you won.";
}


1;
