package TJournal::PFAnalysis::ProfitFactor;

# PFAnalysis.pm -- author: Oliver Bossert
# Copyright (C) 2004 Oliver Bossert
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;

use TJournal::Conf;
use TJournal::DB;

use TJournal::PFAnalysis;
our @ISA = qw(TJournal::PFAnalysis);

use TJournal::PFAnalysis::AvgGain;
use TJournal::PFAnalysis::AvgLoss;
use TJournal::PFAnalysis::NbGains;
use TJournal::PFAnalysis::NbLosses;

############################################################
sub calc {
############################################################
  my $self = shift;

  my $o1 = TJournal::PFAnalysis::AvgGain->new( $self->{'data'} );
  my $o2 = TJournal::PFAnalysis::NbGains->new( $self->{'data'} );
  my $o3 = TJournal::PFAnalysis::AvgLoss->new( $self->{'data'} );
  my $o4 = TJournal::PFAnalysis::NbLosses->new( $self->{'data'} );

  my $avg_g = $o1->calc();
  my $avg_l = $o3->calc();
  my $nb_g = $o2->calc();
  my $nb_l = $o4->calc();
  my $total = $nb_g + $nb_l;
  return 0 if ($total == 0);
  my $percent_g = $nb_g / $total;
  my $percent_l = $nb_l / $total;

  my $ret = "nan";
  $ret = ($percent_g * $avg_g) / ($percent_l * ($avg_l * -1)) unless ( $avg_l == 0 );

  return $ret;
}

############################################################
sub formated {
############################################################
  my $self = shift;
  my $res = $self->calc();
  $res =  sprintf("%.2f",$res);
  return $res;
}

############################################################
sub color {
############################################################
  my $self = shift;

  my $cal = $self->calc();
  TJournal::Conf::default("PFAnalysis::ProfitFactor::Coloring", 
			  "verybad:0~0.5;bad:0.5~1;neutral:1~2;good:2~3;verygood:3~99999");
  my $line = TJournal::Conf::get("PFAnalysis::ProfitFactor::Coloring");
  my $col = 'neutral';
  foreach my $val ( split /;/, $line ) {
    if ( $val =~ /(\w+):([\d\.-]+)~([\d\.-]+)/ ) {
      if ( $cal >= $2 && $cal < $3 ) {
	$col = $1;
      }
    }
  }
  return $col;
}

############################################################
sub name {
############################################################
  my $self = shift;
  return "Profit Factor";
}

############################################################
sub info {
############################################################
  my $self = shift;
  return "The Profit Factor is calculated as the ratio of the averages of your " .
    "winning and losing trades. \nValues below indicate abad system; values above " .
      "two are good.";
}


1;
