package TJournal::PFAnalysis::SumGain;

# PFAnalysis.pm -- author: Oliver Bossert
# Copyright (C) 2004 Oliver Bossert
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

use strict;

use TJournal::Conf;
use TJournal::DB;

use TJournal::PFAnalysis;
our @ISA = qw(TJournal::PFAnalysis);


############################################################
sub calc {
############################################################
  my $self = shift;

  return "" if (!defined($self->{'data'}->{'trade'}));

  my $res = 0;
  my $trade;

  foreach $trade ( sort {$a->{open}->{date} cmp $b->{open}->{date} } @{$self->{'data'}->{'trade'}} ) {

    my $var  = 0;
    my $diff = 0;

    my $sold = ($trade->{'type'} !~ /short/i) ? $trade->{'close'}->{'value'} : $trade->{'open'}->{'value'};
    $sold *= $trade->{'nb'};
    my $bought = ($trade->{'type'} =~ /short/i) ? $trade->{'close'}->{'value'} : $trade->{'open'}->{'value'};
    $bought *= $trade->{'nb'};
    my $is_long = ($trade->{'type'} !~ /short/i) ? 1 : 0;

    $diff = $sold - $bought - $trade->{'fee'}->{'value'};
    $var = 0;
    if ($is_long) {
      $var = ( $bought != 0 ) ? ( $diff / $bought ) : 0;
    } else {
      $var = ($sold !=0 ) ? ( $diff / $sold ) : 0;
    }

    $res += $diff if ($diff > 0);

  }

  return $res;
}

############################################################
sub formated {
############################################################
  my $self = shift;
  my $res = $self->calc();
  $res =  sprintf("%.2f",$res);
  return $res;
}

############################################################
sub color {
############################################################
  my $self = shift;

  my $cal = $self->calc();
  TJournal::Conf::default("PFAnalysis::WinRatio::CumGain::Coloring", 
			  "verybad:0~1");
  my $line = TJournal::Conf::get("PFAnalysis::WinRatio::CumGain::Coloring");
  my $col = 'neutral';
  foreach my $val ( split /;/, $line ) {
    if ( $val =~ /(\w+):([\d\.-]+)~([\d\.-]+)/ ) {
      if ( $cal >= $2 && $cal < $3 ) {
	$col = $1;
      }
    }
  }
  return $col;
}

############################################################
sub name {
############################################################
  my $self = shift;
  return "Sum of your gains";
}

############################################################
sub info {
############################################################
  my $self = shift;
  return "Your sum of your gains.\n";
}


1;
