#!/usr/bin/perl
#
# ab-convert.pl -- author: Oliver Bossert
# Copyright (C) 2004 Oliver Bossert
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#

use strict;
use warnings;

use Getopt::Long;
use LWP::Simple;
use HTML::TableExtract;

my $year = ( (localtime)[5] + 1900 );
GetOptions ('year=s' => \$year);

my $user = shift;
unless ( defined($user) ) {
  print "ab-convert.pl (c) Oliver Bossert 2004\n";
  print "For more information and licence: http://www.olfsworld.de/software/otj/\n\n";
  print "usage: $0 [options] <user>\n";
  print "       user: username or id\n";
  print "       options:\n";
  print "        --year=$year -- Specify the year\n";
  exit;
}

unless ( $user =~ /^\d+$/ ) {
  my $url = "http://www.aktienboard.com/vb/search.php?do=process&searchuser=" . 
    $user . "&starteronly=0&exactname=1";
  my $content = get( $url );
  die "Could not fetch user $user - no network connection!\n" unless (defined($content));
  if ( $content =~ /member\.php\?s=[\d\w]*&amp;u=(\d+)/m ) {
    $user = $1;
  } else {
    die "Could not find user $user!\nTry to use the member-id!\n";
  }
}

#
# Download the depot
#
my $string;
my $url = "http://www.aktienboard.com/vb/musterdepot.php?".
	      "do=show&mduserid=$user&watchlist=showalltrades&year=$year";
$string = get( $url );
if (!defined($string)) {
  die "Could not fetch the MD! Check your network connection and the user-id!";
}

#
# Extract the data
#
print "# This data was extracted by ab-convert (c) Oliver Bossert 2004\n";
print "# You can import the data in OTJ (Olfs Trading Journal).\n";
print "# This software can be found at: http://www.olfsworld.de/software/otj/\n";
print "# The data was extracted from the following url: $url\n#\n";
my @positions = ();
my $te = new HTML::TableExtract (
				 headers => [qw(Datum
						Order
						Wertpapier
						WKN
						Ort
						Anzahl
						Preis),
						'G / V'],
				);
$te->parse( $string );
foreach my $table ($te->tables() ) {
  foreach my $row ( $te->rows($table) ) {
    $row->[0] =~ s/(\d{2})-(\d{2})-(\d{4}) - (\d{2}):(\d{2})/$3-$2-$1_$4:$5:00/;
    $row->[5] =~ s/[^\d]//g;
    $row->[6] =~ s/[^\d,\.]//g;
    $row->[6] =~ s/,/./g;
    foreach my $r ( @$row ) {
      $r =~ s/[\r\n\t]//g;
    }
    #print join("\t", @$row) . "\n";
    push @positions, $row;
  }
}

# Generate the trades
my %depot = ();
my $id = 0;
print '# ' . join("\t", qw(Id Code Nb direction OpenDate
			   OpenVal CloseDate CloseVal Fee Comment)) . "\n";
foreach my $pos ( sort { $a->[0] cmp $b->[0] } @positions ) {

#  print STDERR "=" x 80 . "\n";
#  print STDERR join("\n", @$pos) . "\n";
#  foreach my $c ( keys %depot ) {
#    print STDERR "  => " . $c . " --> " . $depot{$c}{nb} . "\n";
#    foreach my $a ( @{$depot{$c}{ob}} ) {
#      print STDERR "        => " . join("; ", @{$a}) . "\n";
#    }
#  }
#  print STDERR "=" x 80 . "\n";

  # No stocks of this type in the depot
  if ( !defined($depot{$pos->[2]}{nb}) || 
       $depot{$pos->[2]}{nb} == 0 ) {
    if ( $pos->[1] =~ /Verkauf/ ) {
      $depot{$pos->[2]}{nb} = -$pos->[5];
    } else {
      $depot{$pos->[2]}{nb} = $pos->[5];
    }
    push @{$depot{$pos->[2]}{ob}}, $pos;
  }

  # There are some stocks in the depot ...
  else {

    # Add the stocks
    if ( $pos->[1] =~ /Verkauf/ &&
	 $depot{$pos->[2]}{nb} < 0 ) {
      $depot{$pos->[2]}{nb} -= $pos->[5];
      push @{$depot{$pos->[2]}{ob}}, $pos;
    }
    elsif ( $pos->[1] =~ /^Kauf/ &&
	    $depot{$pos->[2]}{nb} > 0 ) {
      $depot{$pos->[2]}{nb} += $pos->[5];
      push @{$depot{$pos->[2]}{ob}}, $pos;
    }

    # Buy / Sell for 0,00
    elsif ( $pos->[1] =~ /^ausgebucht/ ) {
    }

    # Generate a trade
    else {

      # Go through the trades
      my $nb= $pos->[5];
      if ( $pos->[1] =~ /Verkauf/ ) {

	$depot{$pos->[2]}{nb} -= $pos->[5];

	foreach my $trade ( sort {$a->[0] cmp $b->[0]} @{$depot{$pos->[2]}{ob}} ) {
	  last if ( $nb == 0 );
	  next if ( $trade->[5] == 0 );

	  if ( $trade->[1] =~ /^Kauf/ ) {
	    my $tmp_nb;
	    if ( $trade->[5] >= $nb ) {
	      $tmp_nb = $nb;
	      $trade->[5] -= $nb;
	      $nb = 0;
	    } else {
	      $tmp_nb = $trade->[5];
	      $nb -= $trade->[5];
	      $trade->[5] = 0;
	    }

	    print sprintf("%06d", $id++) . "\t" . $pos->[3] . "\t" . $tmp_nb;
	    print "\tlong";
	    print "\t" . join("\t", ( $trade->[0], # Open Date
				      $trade->[6], # Open Price
				      $pos->[0],
				      $pos->[6],
				      sprintf("%.2f", (0.003*$tmp_nb*$trade->[6] + 0.003*$tmp_nb*$pos->[6])),
				      $pos->[2] . "/" . $pos->[4]
				    )) . "\n";

	  }
	}
      }

      else {

	$depot{$pos->[2]}{nb} += $pos->[5];

	foreach my $trade ( sort {$a->[0] cmp $b->[0]} @{$depot{$pos->[2]}{ob}} ) {
	  last if ( $nb == 0 );
	  next if ( $trade->[5] == 0 );

	  if ( $trade->[1] =~ /^Verkauf/ ) {
	    my $tmp_nb;
	    if ( $trade->[5] >= $nb ) {
	      $tmp_nb = $nb;
	      $trade->[5] -= $nb;
	      $nb = 0;
	    } else {
	      $tmp_nb = $trade->[5];
	      $nb -= $trade->[5];
	      $trade->[5] = 0;
	    }

	    print sprintf("%06d", $id++) . "\t" . $pos->[3] . "\t" . $tmp_nb;
	    print "\tshort";
	    print "\t" . join("\t", ( $trade->[0], # Open Date
				      $trade->[6], # Open Price
				      $pos->[0],
				      $pos->[6],
				      sprintf("%.2f", (0.003*$tmp_nb*$trade->[6] + 0.003*$tmp_nb*$pos->[6])),
				      $pos->[2] . "/" . $pos->[4]
				    )) . "\n";

	  }
	}
      }

      $pos->[5] = $nb;
      push @{$depot{$pos->[2]}{ob}}, $pos;

#      print STDERR ">>>>>" . $pos->[2] . " ==> " . $pos->[5] . " ($nb)\n";

    }
  }
}
